package agent.reducer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

public class TableMapPanel extends JPanel {
	private final WCModel model = new WCModel();
	private final JTable table;

	public static void main(String[] args) {
		JFrame frame = new JFrame("Word Count Table");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		TableMapPanel table = new TableMapPanel();
		table.addEntry("AAA", 123);
		table.addEntry("BBB", 13);
		JScrollPane scrollPane = new JScrollPane(table);;
		frame.getContentPane().add(scrollPane);
		frame.setSize(400,100);
		frame.setVisible(true);
		table.addEntry("CCC", 23);
	}

	public TableMapPanel() {
		super(new BorderLayout());
		table = new JTable(model) {
			private final Color evenColor = new Color(250, 250, 250);
			public Component prepareRenderer(TableCellRenderer tcr, int row, int column) {
				Component c = super.prepareRenderer(tcr, row, column);
				if(isRowSelected(row)) {
					c.setForeground(getSelectionForeground());
					c.setBackground(getSelectionBackground());
				}else{
					c.setForeground(getForeground());
					c.setBackground((row%2==0)?evenColor:getBackground());
				}
				return c;
			}
		};

		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(model);
		table.setRowSorter(sorter);

		TableColumn col = table.getColumnModel().getColumn(0);
		col.setMinWidth(200);
		col.setMaxWidth(250);
		col = table.getColumnModel().getColumn(1);
		col.setMinWidth(50);
		col.setMaxWidth(100);
		col.setResizable(false);
		table.setAutoCreateRowSorter(true);
		table.setFillsViewportHeight(true);
		add(new JScrollPane(table));
		setPreferredSize(new Dimension(320, 200));
	}

	public void addEntry(String key, Object obj) {
		model.addEntry(key, obj);
	}

	public void removeEntry(String key) {
		model.removeEntry(key);
	}
	
	public void clear() {
//		System.out.println("TableMapPanel: model.getRowCount() = "+model.getRowCount());
		for (int i = 0 ; i < model.getRowCount() ; i++ ) {
			model.removeRow(i);
		}
	}
}

class WCModel extends DefaultTableModel {
	private Map map = null;
	public WCModel() {
		map = new HashMap();
	}
	private static final ColumnContext[] columnArray = {
		new ColumnContext("Key",   String.class,  true),
		new ColumnContext("Value", Integer.class,  true)
	};
	public void addEntry(String key, Object value) {
		map.put(key, value);
		Object[] obj = {key, value};
		super.addRow(obj);
	}
	public void removeEntry(String key) {
		if (map.containsKey(key)) {
			map.remove(key);
		}
	}
	public boolean isCellEditable(int row, int col) {
		return columnArray[col].isEditable;
	}
	public Class<?> getColumnClass(int modelIndex) {
		return getValueAt(0, modelIndex).getClass();
	}
	public int getColumnCount() {
		return columnArray.length;
	}
	public String getColumnName(int modelIndex) {
		return columnArray[modelIndex].columnName;
	}
	private static class ColumnContext {
		public final String  columnName;
		public final Class   columnClass;
		public final boolean isEditable;
		public ColumnContext(String columnName, Class columnClass, boolean isEditable) {
			this.columnName = columnName;
			this.columnClass = columnClass;
			this.isEditable = isEditable;
		}
	}
}
