package agent.wordcount;

//-*- mode:java; encoding:utf8n; coding:utf-8 -*-
//vim:set fileencoding=utf-8:
//http://terai.xrea.jp/Swing/AddRow.html
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;

public class TableComponent extends JPanel {
	private final KVModel model = new KVModel();

	private final JTable table;
	public TableComponent() {
		super(new BorderLayout());
		table = new JTable(model) {
			private final Color evenColor = new Color(250, 250, 250);
			public Component prepareRenderer(TableCellRenderer tcr, int row, int column) {
				Component c = super.prepareRenderer(tcr, row, column);
				if(isRowSelected(row)) {
					c.setForeground(getSelectionForeground());
					c.setBackground(getSelectionBackground());
				}else{
					c.setForeground(getForeground());
					c.setBackground((row%2==0)?evenColor:getBackground());
				}
				return c;
			}
		};

		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(model);
		table.setRowSorter(sorter);

		TableColumn col = table.getColumnModel().getColumn(0);
		col.setMinWidth(200);
		col.setMaxWidth(250);
		col = table.getColumnModel().getColumn(1);
		col.setMinWidth(50);
		col.setMaxWidth(50);
		col.setResizable(false);

		model.addEntry(new KVEntry("Name 1", new Integer(10)));
		model.addEntry(new KVEntry("Name 2", new Integer(1)));
		model.addEntry(new KVEntry("Name d", new Integer(3)));
		model.addEntry(new KVEntry("Name c", new Integer(20)));
		model.addEntry(new KVEntry("Name b", new Integer(40)));
		model.addEntry(new KVEntry("Name a", new Integer(13)));
		model.addEntry(new KVEntry("Name 0", new Integer(22)));

		table.setAutoCreateRowSorter(true);
		table.setFillsViewportHeight(true);
		table.setComponentPopupMenu(new TablePopupMenu());
		add(new JScrollPane(table));
		setPreferredSize(new Dimension(320, 200));
	}

	class TestCreateAction extends AbstractAction{
		public TestCreateAction(String label, Icon icon) {
			super(label,icon);
		}
		public void actionPerformed(ActionEvent evt) {
			testCreateActionPerformed(evt);
		}
	}
	private void testCreateActionPerformed(ActionEvent e) {
		model.addEntry(new KVEntry("New name", new Integer(0)));
	}

	class DeleteAction extends AbstractAction{
		public DeleteAction(String label, Icon icon) {
			super(label,icon);
		}
		public void actionPerformed(ActionEvent evt) {
			deleteActionPerformed(evt);
		}
	}
	public void deleteActionPerformed(ActionEvent evt) {
		int[] selection = table.getSelectedRows();
		if(selection==null || selection.length<=0) return;
		for(int i=selection.length-1;i>=0;i--) {
			model.removeRow(table.convertRowIndexToModel(selection[i]));
		}
	}

	private class TablePopupMenu extends JPopupMenu {
		private final Action deleteAction = new DeleteAction("delete", null);
		public TablePopupMenu() {
			super();
			add(new TestCreateAction("add", null));
			//add(new ClearAction("clearSelection", null));
			addSeparator();
			add(deleteAction);
		}
		public void show(Component c, int x, int y) {
			int[] l = table.getSelectedRows();
			deleteAction.setEnabled(l!=null && l.length>0);
			super.show(c, x, y);
		}
	}

	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}
	public static void createAndShowGUI() {
		try{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}catch(Exception e) {
			e.printStackTrace();
		}
		JFrame frame = new JFrame("AddRow");
		frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		frame.getContentPane().add(new TableComponent());
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}
}

class KVModel extends DefaultTableModel {
	private static final ColumnContext[] columnArray = {
		new ColumnContext("Key",   String.class,  true),
		new ColumnContext("Value", Integer.class,  true)
	};
	public void addEntry(KVEntry t) {
		Object[] obj = {t.getKey(), t.getValue()};
		super.addRow(obj);
	}
	public boolean isCellEditable(int row, int col) {
		return columnArray[col].isEditable;
	}
	public Class<?> getColumnClass(int modelIndex) {
		return getValueAt(0, modelIndex).getClass();
	}
	public int getColumnCount() {
		return columnArray.length;
	}
	public String getColumnName(int modelIndex) {
		return columnArray[modelIndex].columnName;
	}
	private static class ColumnContext {
		public final String  columnName;
		public final Class   columnClass;
		public final boolean isEditable;
		public ColumnContext(String columnName, Class columnClass, boolean isEditable) {
			this.columnName = columnName;
			this.columnClass = columnClass;
			this.isEditable = isEditable;
		}
	}
}
/*
class TestModel extends DefaultTableModel {
	private static final ColumnContext[] columnArray = {
		new ColumnContext("No.",   Integer.class, false),
		new ColumnContext("Key",   String.class,  true),
		new ColumnContext("Value", Integer.class,  true)
	};
	private int number = 0;
	public void addTest(KVEntry t) {
		Object[] obj = {number, t.getKey(), t.getValue()};
		super.addRow(obj);
		number++;
	}
	public boolean isCellEditable(int row, int col) {
		return columnArray[col].isEditable;
	}
	public Class<?> getColumnClass(int modelIndex) {
		return getValueAt(0, modelIndex).getClass();
	}
	public int getColumnCount() {
		return columnArray.length;
	}
	public String getColumnName(int modelIndex) {
		return columnArray[modelIndex].columnName;
	}
	private static class ColumnContext {
		public final String  columnName;
		public final Class   columnClass;
		public final boolean isEditable;
		public ColumnContext(String columnName, Class columnClass, boolean isEditable) {
			this.columnName = columnName;
			this.columnClass = columnClass;
			this.isEditable = isEditable;
		}
	}
}
*/

class KVEntry{
	private String key = null;
	private Integer value = 0;
	public KVEntry(String key, Integer value) {
		this.key = key;
		this.value = value;
	}
	public void setKey(String key) {
		this.key = key;
	}
	public void setValue(Integer value) {
		this.value = value;
	}
	public String getKey() {
		return key;
	}
	public Integer getValue() {
		return value;
	}
}
