package agent.wordcount;

import java.awt.Rectangle;
import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import agentspace.Agent;
import agentspace.AgentIdentifier;
import agentspace.Context;
import agentspace.InvalidURLException;
import agentspace.Message;
import agentspace.Mobile;
import agentspace.event.AgentEvent;

public class WordCountAgent implements Agent, Mobile {
	Map map = null;
	Context context;
	transient WordCountWindow window = null;
	
	String urlString = null;
	URL homeURL = null;
	String pattern = null;
	String path = "/Users/ichiro/work/as6/test.txt";
	Rectangle rect = null;
	URL targetURL = null;
	String result = "";
	String NAME = "WordCount";	
	boolean searching = false;
	
	public void create(AgentEvent evt, Context context) {
		this.context = context;
		context.setName(NAME);
		System.out.println("WordCount: method create() called");
		if (homeURL == null) {
			homeURL = context.getHostURL();
		}
		window = new WordCountWindow(this, path);
		window.setTable(true);
		window.setVisible(true);
		this.context = context;
	}

	/* 
	 * @see agentspace.SerialiableAgent#destroy(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void destroy(AgentEvent evt, Context context) {
		System.out.println("WordCount: method destroy() called");
		if (window != null) {
			window.setVisible(false);
		}
	}
	/* 
	 * @see agentspace.SerialiableAgent#suspend(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void suspend(AgentEvent evt, Context context) {
		System.out.println("WordCount: method suspend() called");
		if (window != null) {
			window.setVisible(false);
		}
	}

	/* 
	 * @see agentspace.SerialiableAgent#resume(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void resume(AgentEvent evt, Context context) {
		System.out.println("WordCount: method resume() called");
		window = new WordCountWindow(this, path);
		window.setTable(true);
		window.setVisible(true);
		this.context = context;
	}
	/* 
	 * @see agentspace.MobileAgent#arrive(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void arrive(AgentEvent evt, Context context) {
		this.context = context;
		System.out.println("WordCount: method arrive() called");
//		System.out.println("WordCount: home = "+homeURL);
//		System.out.println("WordCount: current = "+context.getHostURL());
		InetAddress	homeAddr;
		try {
			homeAddr = InetAddress.getByName(homeURL.getHost());
			InetAddress	currentAddr = InetAddress.getByName(context.getHostURL().getHost()); 
			if (homeAddr.equals(currentAddr) && 
					homeURL.getPort() == context.getHostURL().getPort() && searching ) { 
				searching = false;
//				System.out.println("WordCount: arriving at the home");
				window = new WordCountWindow(this, path);
				window.setTable(true);
				window.setMap(map);
				window.setTable(true);
				window.setBounds(rect);
				window.setVisible(true);
				
				AgentIdentifier[] aids = context.getAgents("Reducer");
				for (int i = 0 ; i < aids.length ; i++ ) {
					Message msg = new Message("addMap");
					msg.setArg(map);
					context.invoke(aids[i], msg);
//					System.out.println("WordCount: context.invoke("+aids[i]+", "+msg+") called");
				}
				
			}
			else {
//				System.out.println("WordCount: arriving at remote host");
				WordCount wc = new WordCount(path);
				Map current = wc.getMap();
				if (map != null) {
					MapUtil.merge(current, map);
				}
				else {
					map = current; 
				}
//				result = compare(path, pattern);
				searching = true;
				context.move(homeURL);
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}
		
	/* 
	 * @see agentspace.MobileAgent#leave(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void leave(AgentEvent evt, Context context) {
		System.out.println("WordCount: method leave() called");
		if (window != null) {
			rect = window.getBounds();
			window.setVisible(false);
			window = null;
		}
	}
	
	public void go(URL targetURL, String path) throws IOException, InvalidURLException {
		this.targetURL = targetURL;
		this.path = path;
		context.move(targetURL);
	}
	
	public void go() {
		try {
//			System.out.println("WordCountAgent: go");
			context.move(targetURL);
		} 
		catch (InvalidURLException e) {
			e.printStackTrace();
		}
		catch (IOException ex) {}
	}

	public Map getMap() {
		return map;
	}
	
	public void setTarget(String urlString) {
		this.urlString = urlString;		
		try {
			targetURL = new URL(urlString);
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		window.setTargetURL(urlString);
	}

	public void setPath(String path) {
		this.path = path;
		window.setPath(path);
	}
}

