package agent.wordcount;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import agentspace.InvalidURLException;

public class WordCountWindow extends JFrame {
	WordCountAgent agent;
	URL homeURL = null;
	URL targetURL = null;
	Map map = null;
	JTextArea area;
	ActionListener listener;
	JPanel wordCountPanel;
	JPanel networkPanel;
	JTextField pathField;
	JTextField addrField;
	JComponent comp = null;
	JButton goButton;
	JButton clearButton;
	String path = "agent/grep/Grep.java";
	String urlString = "matp://localhost:7000";
	transient TableMapPanel table = null;
	
	public WordCountWindow(WordCountAgent agent, String path) {
		super("Mobile Word Count");
		this.agent = agent;
		this.path = path;
		listener = new GrepAction();
		networkPanel = new JPanel();
		addrField = new JTextField(urlString, 20);
		addrField.setBorder(new TitledBorder("Target URL:"));
		networkPanel.add(addrField);
		goButton = new JButton("Go");
		goButton.setActionCommand("Go");
		goButton.addActionListener(listener);
		networkPanel.add(goButton);
		clearButton = new JButton("Clear");
		clearButton.setActionCommand("Clear");
		clearButton.addActionListener(listener);
		networkPanel.add(clearButton);

		wordCountPanel = new JPanel();
		pathField = new JTextField(path, 33);
		pathField.setBorder(new TitledBorder("Target file(s):"));
		wordCountPanel.add(pathField);
		
		if (map == null) {
			map = new HashMap();
		}
//		tableComp = update(map);
		/*
		area = new JTextArea(30, 5);
		JScrollPane scrollpane = new JScrollPane(area);
		*/
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(networkPanel, BorderLayout.NORTH);
		getContentPane().add(wordCountPanel, BorderLayout.CENTER);
		setTable(true);
		setSize(420, 350);
	}

	public void setTable(boolean flag) {
		if (flag) {
			comp = setMap(map);
			getContentPane().add(comp, BorderLayout.SOUTH);
		}
		else {
			getContentPane().remove(comp);
		}
	}

	public JComponent setMap(Map mapdata) {
		JComponent scrollpane;
		if (table == null) {
			table = new TableMapPanel();
		}
		scrollpane = new JScrollPane(table);
		Iterator it = mapdata.keySet().iterator();
		while (it.hasNext()) {
			String key = (String)it.next();
			Object value = mapdata.get(key);
//			System.out.println("WordCountWindow: key = "+key+", value = "+value);
			table.addEntry(key, value);
		}
		return (JComponent)scrollpane; 
	}
	public void appendText(Map map) {
		this.map = map;
	}

	public void appendText(String result) {
		String content = area.getText();
		area.setText(result+"\n"+content);
	}
	
	class GrepAction implements ActionListener, Serializable {
		public void actionPerformed(ActionEvent evt) {
			String action = evt.getActionCommand();
			if (action.equals("Go")) {
				try {
//					pattern = patternField.getText();
					path = pathField.getText();
					targetURL = new URL(addrField.getText());
//					System.out.println("Grep: targetURL = "+targetURL);
					
					agent.go(targetURL, path);
				} 
				catch (MalformedURLException e) {
					e.printStackTrace();
				} 
				catch (IOException e) {
					e.printStackTrace();
				} 
				catch (InvalidURLException e) {
					e.printStackTrace();
				}
			}
			if (action.equals("Clear")) {
				map.clear();
				setTable(true);
				setVisible(true);
			}
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		WordCountWindow wcw = new WordCountWindow(null, "Users/ichiro/work/as6/test.txt");
//		WordCountWindow wcw = new WordCountWindow(null, args[0]);
		wcw.setVisible(true);
	}

	public void setTargetURL(String urlString) {
		this.urlString = urlString;
		addrField.setText(urlString);
	}

	public void setPath(String path) {
		this.path = path;
		pathField.setText(path);
	}

}
