/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/4
 */

package agentspace;

import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;

import agentspace.context.ContextManager;
import agentspace.network.AgentSocketServer;
import agentspace.registry.AgentRegistry;
import agentspace.runtime.AgentRuntimeManager;
import agentspace.runtime.AgentRuntimeManagerImpl;
import agentspace.shell.AgentShell;

public class AgentServer {
	final static public int SERVER_PORT = 7000;
	static InetAddress addr;
	static String host = null;
	static int port;
	static URL targetURL = null;

	static {
		URL.setURLStreamHandlerFactory(new agentspace.network.AgentURLStreamHandlerFactory());
	}
	
	public static void main(String[] args) {
		String command = null;
		port = SERVER_PORT;
		for (int i = 0; i < args.length; i++) {
			if ((args[i].equals("-port") || args[i].equals("-p"))
					&& (i + 1 < args.length)) {
				try {
					port = new Integer(args[++i]).intValue();
				} 
				catch (NumberFormatException ex) {
					System.exit(1);
				}
			} 
			else if (args[i].equals("-target") && (i + 1 < args.length)) {
				try {
					targetURL = new URL(args[++i]);
				} 
				catch (MalformedURLException e) {
					e.printStackTrace();
				}
			} 
		}
		if (targetURL == null) {
			try {
				targetURL = new URL("matp://localhost:"+SERVER_PORT);
			} 
			catch (MalformedURLException e) {
				e.printStackTrace();
			}
		}
		AgentServer as = new AgentServer(command, port);
	}
	
	AgentServer(String command, int port) {
		try {
			addr = InetAddress.getLocalHost();
			host = addr.getHostAddress();
		} catch (UnknownHostException e) {
			e.printStackTrace();
			System.exit(0);
		}
		System.out.println("AgentServer starting: "+addr+":"+port);
		if (command == null || command.length() == 0) {
			try {
				AgentRegistry registry = new AgentRegistry();
				AgentRuntimeManager manager = new AgentRuntimeManagerImpl(registry);
				ContextManager contextManager = new ContextManager(manager, registry);
				agentspace.monitor.AgentView view = new agentspace.monitor.AgentView(contextManager);
				manager.addAgentRuntimeListener(view);
				AgentShell shell = new AgentShell(contextManager, command);
				shell.start();

				AgentSocketServer server = new AgentSocketServer(manager, port);
				server.start();
			}
			catch (IOException ex) {
				ex.printStackTrace();                
			}
		}
	}
	
	static public int getDefaultPort() {
		return port;
	}
	
	static public String getLocalHost() {
		return host;
	}

	static public URL getTargetURL() {
		return targetURL;
	}
}