/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/4
 */

package agentspace;

import java.io.Serializable;
import java.util.Vector;

public class Message implements Serializable, Cloneable {
	  private String theName = null;
	  public Vector args  = null;
	  public Vector types = null;
	  protected int size = 0;

	  private Future theFuture = null;
	  private int theTimeLimit = 0;
	  private Object theSource = null;
	  private Object theTarget = null;
	  private Exception theException = null;
	  private boolean theSynchronized = false;
	  private boolean theSequential = false;
	  private boolean thePushed = false;
	  private transient Thread theThread = null;
//	  private transient Handler theHandler = null;

	  static final byte TYPE_OBJECT = 0;
	  static final byte TYPE_BOOLEAN = 1;
	  static final byte TYPE_BYTE = 2;
	  static final byte TYPE_SHORT = 3;
	  static final byte TYPE_CHAR = 4;
	  static final byte TYPE_INT = 5;
	  static final byte TYPE_LONG = 6;
	  static final byte TYPE_FLOAT = 7;
	  static final byte TYPE_DOUBLE = 8;
	  static final byte TYPE_VOID = 9;

	  static final byte TYPE_ARRAY_OBJECT = 10;
	  static final byte TYPE_ARRAY_BOOLEAN = 11;
	  static final byte TYPE_ARRAY_BYTE = 12;
	  static final byte TYPE_ARRAY_SHORT = 13;
	  static final byte TYPE_ARRAY_CHAR = 14;
	  static final byte TYPE_ARRAY_INT = 15;
	  static final byte TYPE_ARRAY_LONG = 16;
	  static final byte TYPE_ARRAY_FLOAT = 17;
	  static final byte TYPE_ARRAY_DOUBLE = 18;
	  static final byte TYPE_ARRAY_VOID = 19;

	  public Message(String msg) {
	    args = new Vector();
	    types = new Vector();
	    theName = msg;
	  }

	  public void setArg(Object obj) {
	    types.addElement(new Byte(TYPE_OBJECT));
	    args.addElement(obj);
	    size = size + 1;
	  }

	  public void setArg(boolean value) {
	    types.addElement(new Byte(TYPE_BOOLEAN));
	    args.addElement(new Boolean(value));
	    size = size + 1;
	  }

	  public void setArg(byte value) {
	    types.addElement(new Byte(TYPE_BYTE));
	    args.addElement(new Byte(value));
	    size = size + 1;
	  }

	  public void setArg(short value) {
	    types.addElement(new Byte(TYPE_SHORT));
	    args.addElement(new Short(value));
	    size = size + 1;
	  }

	  public void setArg(char value) {
	    types.addElement(new Byte(TYPE_CHAR));
	    args.addElement(new Character(value));
	    size = size + 1;
	  }

	  public void setArg(int value) {
	    types.addElement(new Byte(TYPE_INT));
	    args.addElement(new Integer(value));
	    size = size + 1;
	  }

	  public void setArg(long value) {
	    types.addElement(new Byte(TYPE_LONG));
	    args.addElement(new Long(value));
	    size = size + 1;
	  }

	  public void setArg(float value) {
	    types.addElement(new Byte(TYPE_FLOAT));
	    args.addElement(new Float(value));
	    size = size + 1;
	  }

	  public void setArg(double value) {
	    types.addElement(new Byte(TYPE_DOUBLE));
	    args.addElement(new Double(value));
	    size = size + 1;
	  }

	  public void setArg(Object obj, int loc) {
	    args.setElementAt(obj, loc);
	  }

	  public Object getArg(int loc) {
	    return args.elementAt(loc);
	  }

	  public int getArgCount() {
	    return size;
	  }

	  public Object[] getArgs() {
	    Object[] objects = new Object[size];
	    for (int i = 0; i < size; i++) {
	      objects[i] = args.elementAt(i);
	    }
	    return objects;
	  }
	   
	  public Class[] getTypes() {
	    Class[] classes = new Class[size];
	    for (int i = 0; i < size; i++) {
	      switch (((Byte)types.elementAt(i)).byteValue()) {
	      case TYPE_OBJECT:
		classes[i] = args.elementAt(i).getClass();
		break;
	      case TYPE_BOOLEAN:
		classes[i] = java.lang.Boolean.TYPE;
		break;
	      case TYPE_BYTE:
		classes[i] = java.lang.Byte.TYPE;
		break;
	      case TYPE_SHORT:
		classes[i] = java.lang.Short.TYPE;
		break;
	      case TYPE_CHAR:
		classes[i] = java.lang.Character.TYPE;
		break;
	      case TYPE_INT:
		classes[i] = java.lang.Integer.TYPE;
		break;
	      case TYPE_LONG:
		classes[i] = java.lang.Long.TYPE;
		break;
	      case TYPE_FLOAT:
		classes[i] = java.lang.Float.TYPE;
		break;
	      case TYPE_DOUBLE:
		classes[i] = java.lang.Double.TYPE;
		break;
	      case TYPE_VOID:
		classes[i] = java.lang.Void.TYPE;
		break;
	      case TYPE_ARRAY_INT:
		classes[i] = java.lang.Byte.TYPE;
	      default:
		throw new UnknownError("Invalid parameter type");	
	      }
	    }
	    return classes;
	  }

	  public void setName(String name) {
	    theName = name;
	  }

	  public String getName() {
	    return theName;
	  }

	  public void setThread(Thread t) {
	    theThread = t;
	  }

	  public Thread getThread() {
	    return theThread;
	  }

	  public void setSource(Object obj) {
	    theSource = obj;
	  }

	  public Object getSource() {
	    return theSource;
	  }

	  public void setTarget(Object obj) {
	    theTarget = obj;
	  }

	  public Object getTarget() {
	    return theTarget;
	  }

	  public void setFuture(Future futureObject) {
	    theFuture = futureObject;
	  }

	  public Future getFuture() {
	    if (theFuture == null) {
	      theFuture = new Future();
	    }
	    return theFuture;
	  }

	  public void setTimelimit(int timelimit) {
	    theTimeLimit = timelimit;
	  }

	  public int getTimelimit() {
	    return theTimeLimit;
	  }
/*
	  public void setHandler(Handler handler) {
	    theHandler = handler;
	  }

	  public Handler getHandler() {
	    return theHandler;
	  }
*/
	  public void setException(Exception e) {
	    if (theException == null) {
	      theException = e;
	    }
	  }

	  public Exception getException() {
	    return theException;
	  }

	  public void setSynchronized(boolean b) {
	    theSynchronized = b;
	  }

	  public boolean isSynchronized() {
	    return theSynchronized;
	  }

	  public void setSequential(boolean b) {
	    theSequential = b;
	  }

	  public boolean getSequential() {
	    return theSequential;
	  }

	  public boolean isSequential() {
	    return theSequential;
	  }

	  public void setPushed(boolean b) {
	    thePushed = b;
	  }

	  public boolean isPushed() {
	    return thePushed;
	  }

	  public String toString() {
	    String s = "[" + getName() + ":";
	    for (int i = 0 ; i < getArgCount() ; i++ ) {
	      Object obj = getArg(i);
	      s = s + ", " + obj.getClass();
	    }
	    s = s + "]";
	    return s;
	  }
}
