/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/12
 */


package agentspace.monitor;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;

import agentspace.AgentServer;
import agentspace.context.ContextManager;
import agentspace.monitor.splash.AgentSplashWindow;
import agentspace.monitor.table.AgentTableControl;
import agentspace.monitor.table.AgentTableToolBar;
import agentspace.monitor.table.AgentTableView;
import agentspace.runtime.event.AgentRuntimeEvent;
import agentspace.runtime.event.AgentRuntimeListener;

class WindowClosing extends WindowAdapter {
	public void windowClosing(WindowEvent e) {
		System.exit(0);
	}
}

public class AgentView extends JFrame implements AgentRuntimeListener {
	AgentTableView list;
	
	public AgentView(ContextManager manager) {
		super("AgentSpace: System Monitor ("+AgentServer.getLocalHost()+":"+AgentServer.getDefaultPort()+")");
		
		AgentSplashWindow w = new AgentSplashWindow(new ImageIcon(
				"agentspace/monitor/images/logo.gif"), 5000);
	    
	    addWindowListener(new WindowClosing());

		AgentTableControl listControl = new AgentTableControl(manager, this);
		AgentTableToolBar listToolBar = new AgentTableToolBar(listControl);
		list = new AgentTableView(listControl, listToolBar);

		JPanel listTab = new JPanel();
		listTab.setLayout(new BorderLayout());
		listTab.add(listToolBar, BorderLayout.NORTH);
		listTab.add(list, BorderLayout.CENTER);

		getContentPane().add(listToolBar, BorderLayout.NORTH);
		getContentPane().add(list, BorderLayout.CENTER);

		setSize(600, 200);
		setVisible(true);
	}

	/* (non-Javadoc)
	 * @see agentspace.runtime.event.AgentRuntimeListener#agentAdded(agentspace.runtime.event.AgentRuntimeEvent)
	 */
	public void agentAdded(AgentRuntimeEvent evt) {
		list.addElement(evt.getAgentContainer());
	}

	/* (non-Javadoc)
	 * @see agentspace.runtime.event.AgentRuntimeListener#agentRemoved(agentspace.runtime.event.AgentRuntimeEvent)
	 */
	public void agentRemoved(AgentRuntimeEvent evt) {
		list.removeElement(evt.getAgentContainer());
	}

	/* (non-Javadoc)
	 * @see agentspace.runtime.event.AgentRuntimeListener#agentAdded(agentspace.runtime.event.AgentRuntimeEvent)
	 */
	public void agentChanged(AgentRuntimeEvent evt) {
		list.changeElement(evt.getAgentContainer());
	}

}