/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/13
 */

package agentspace.monitor.table;

import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import agentspace.runtime.AgentContainer;
import agentspace.runtime.AgentStatus;

public class AgentTableModel extends DefaultTableModel {
	final String[] names = { "Name", "Lifecycle", "Status", "Identifier", "Type", "Date" };
	final int[] lengths = {50, 30, 30, 100, 30, 100};
	private List list = null;

	JTable theTable = null;

	public AgentTableModel() {
		super();
		list = new Vector();
		for (int i = 0 ; i < names.length ; i++ ) {
			super.addColumn(names[i]);
		}
	}
	
	public synchronized void setTable(JTable table) {
		theTable = table;
		for (int i = 0; i < names.length; i++) {
			TableColumn column = table.getColumn(names[i]);
			column.setPreferredWidth(lengths[i]);
		}
	}

	public JTable getTable() {
		return theTable;
	}

	public synchronized int getSelectedIndex() {
		int[] rows = getTable().getSelectedRows();
		if (rows != null && rows.length > 0) {
			return rows[0];
		} else {
			return -1;
		}
	}

	public synchronized String getName(int row) {
		if (0 <= row && row < getRowCount()) {
			return (String) getValueAt(row, 0);
		}
		return null;
	}

	public synchronized void addElement(AgentContainer ac) {
		list.add(ac);
		Vector rowlist = new Vector();
		rowlist.addElement(ac.getName());
		rowlist.addElement(AgentStatus.getLifeName(ac.getStatus()));
		rowlist.addElement(AgentStatus.getProcessName(ac.getStatus()));
		rowlist.addElement(ac.getIdentifier().toString());
		rowlist.addElement(ac.getType());
		rowlist.addElement(new Date(ac.getTime()));
		super.addRow(rowlist);
	}

	public synchronized void insertElementAt(AgentContainer ac, int row) {
		list.add(ac);
		Vector rowlist = new Vector();
		rowlist.addElement(ac.getName());
		rowlist.addElement(AgentStatus.getLifeName(ac.getStatus()));
		rowlist.addElement(AgentStatus.getProcessName(ac.getStatus()));
		rowlist.addElement(ac.getIdentifier().toString());
		rowlist.addElement(ac.getType());
		rowlist.addElement(new Date(ac.getTime()));
		super.insertRow(row, rowlist);
	}

	public synchronized void removeElement(AgentContainer ac) {
		int row = list.indexOf(ac);
		removeElementAt(row);
	}

	public synchronized void removeElementAt(int row) throws ArrayIndexOutOfBoundsException {
		if (0 <= row && row < getRowCount()) {
			super.removeRow(row);
			list.remove(row);
		}
		else {
			throw new ArrayIndexOutOfBoundsException("Illegal row specified: "+row+" agent list table ");
		}
	}

	public synchronized void changeElement(AgentContainer ac) {
		int row = list.indexOf(ac);
		replaceElementAt(ac, row);
	}

	public synchronized void replaceElementAt(AgentContainer ac, int row) throws ArrayIndexOutOfBoundsException {
		if (0 <= row && row < getRowCount()) {
//			System.out.println("AgentTableControl: replaceElementAt(" + ac + ", "+ row + ")");
			list.set(row, ac);
			super.setValueAt(ac.getName(), row, 0);
			super.setValueAt(AgentStatus.getLifeName(ac.getStatus()), row, 1);
			super.setValueAt(AgentStatus.getProcessName(ac.getStatus()), row, 2);
			super.setValueAt(ac.getIdentifier().toString(), row, 3);
			super.setValueAt(ac.getType(), row, 4);
			super.setValueAt(new Date(ac.getTime()), row, 5);
		}
		else {
			throw new ArrayIndexOutOfBoundsException("Illegal row specified: "+row+" agent list table ");
		}
	}

	public synchronized AgentContainer getElementAt(int row) {
		if (0 <= row && row < getRowCount()) {
			return (AgentContainer) list.get(row);			
		}
		else {
			return null;
		}
	}
}

