/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/6
 */

package agentspace.network;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import agentspace.network.protocol.Request;
import agentspace.network.protocol.RequestParser;
import agentspace.network.protocol.Response;
import agentspace.runtime.AgentRuntimeManager;

public class AgentServerHandler implements Runnable {
	protected final static int BUF_SIZE = 2048;
	protected Socket clientSocket;
	protected int timeout = 0;
	protected InputStream clientInputStream = null;
	protected OutputStream clientOutputStream = null;
	protected AgentRuntimeManager manager = null;
	
	AgentServerHandler(AgentRuntimeManager manager, Socket clientSocket, int timeout) {
		this.manager = manager;
	    this.clientSocket = clientSocket;
	    this.timeout = timeout;
	}
	
	public synchronized void run() {
		try {
			if (clientSocket != null) {
				clientOutputStream = new BufferedOutputStream(clientSocket.getOutputStream()); 
				clientInputStream = new BufferedInputStream(clientSocket.getInputStream()); 
				clientSocket.setSoTimeout(timeout);
				clientSocket.setTcpNoDelay(true);
				handleProtocol(clientInputStream, clientOutputStream);
				clientOutputStream.close();
				clientInputStream.close();
				clientSocket.close();
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		try {
			clientSocket.close();
		}
		catch (IOException ex) {}
	}

	protected boolean handleProtocol(InputStream in, OutputStream os) throws IOException {
		AgentProtocolHandler handler = null;
	    try {
	    	RequestParser request = new RequestParser(in);
//	    	System.out.println("AgentServerHandler: request = "+request);
	    	String protocol = request.getProtocol();
	    	String className = "agentspace.network.protocol." + protocol.toLowerCase() + ".SessionHandler";
	    	Object obj = Class.forName(className).newInstance();
	    	if (obj != null && obj instanceof AgentProtocolHandler) {
				handler = (AgentProtocolHandler)obj;
			    handler.setRequest((Request)request);				
		    	handler.setAgentRuntimeManager(manager);
		    	Response response = handler.handle();
//		    	System.out.println("AgentServerHandler: response = "+response);
		    	os.write(response.toByteArray());
		    	os.flush();
		    	return true;
	    	}
	    	else {
	    		throw new IOException("non-supported protocol: "+protocol);
	    	}
	    }
	    catch (ClassNotFoundException e) {
	    	e.printStackTrace();
	    }
	    catch (InstantiationException e) {
	    	e.printStackTrace();
	    }
	    catch (IllegalAccessException e) {
	    	e.printStackTrace();
	    }
	    return false;
	}
}
