/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.StringTokenizer;

public class ResponseParser extends Response {
    protected Hashtable attributes = null;

    public ResponseParser(InputStream in) throws IOException {
        int contentLength = 0;
        String statusLine = Util.readLine(in);
        StringTokenizer tokens = new StringTokenizer(statusLine, " ");
        if (tokens.countTokens() < 2) {
        	throw new IOException("Invalid Response: " + statusLine);
        }
        String protocol = tokens.nextToken();
        if (protocol == null || protocol.indexOf('/') < 0) {
        	throw new IOException("Invalid Response: " + statusLine);
        }
        StringTokenizer st = new StringTokenizer(protocol, "/"); 		
        responseProtocol = st.nextToken().trim().toUpperCase();
        responseVersion = st.nextToken().trim();
        String code = tokens.nextToken().trim();
        if (code == null) {
        	throw new IOException("Invalid Response: " + statusLine);
        }
        try {
            responseCode = new Integer(code).intValue();
        }
        catch (NumberFormatException ex) {
            responseCode = -1;
        }
        responseComment = tokens.nextToken().trim();
        while (true) {
            String attributeLine = Util.readLine(in);
            if (attributeLine == null || attributeLine.length() == 0) {
                break;
            }
            if (attributes == null) {
                attributes = new Hashtable();
            }
            int len = attributeLine.length();
            int pos = attributeLine.indexOf(":");
            if (pos > 0) {
                String name = attributeLine.substring(0, pos).trim();
                String property = attributeLine.substring(pos + 1, len).trim();
                attributes.put(name, property);
                if (name.equalsIgnoreCase("Content-Length")) {
                    try {
                        contentLength = new Integer(property).intValue();
                    }
                    catch (NumberFormatException ex) {}
                }
            }
            else {
            	throw new IOException("Invalid Response: " + statusLine);
            }
        }
        if (contentLength > 0) {
            java.io.ByteArrayOutputStream baos = new java.io.ByteArrayOutputStream();
            for (int i = 0 ; i < contentLength ; i++ ) {
                int result = in.read();
                if (result < 0) {
                    throw new IOException("Invalid Content");
                }
                baos.write((byte)result);
            }
            baos.flush();
            responseContent = baos.toByteArray();
            baos.close();
        }
    }
}
