/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol.http;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import agentspace.network.protocol.Request;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseFactory;
import agentspace.runtime.AgentRuntimeManager;

public class SessionHandler implements agentspace.network.AgentProtocolHandler {
	AgentRuntimeManager manager;
	Request request;

	public SessionHandler() {
		super();
	}

	public void setRequest(Request request) {
		this.request = request;
	}

	public Response handle() {
		Response response = null;
		String method = request.getMethod();
		System.out.println("http.RequestHandler: "+request);
		if (method.equals("GET")) {
			response = handleGETMethod();
		} else {
			response = handleUnknownMethod(method);
		}
		return response;
	}

	protected Response handleGETMethod() {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("HTTP");
		response.setVersion("1.0");
		try {
			byte[] data = null;
			String fname = request.getURI();
			if (fname.equals("/SYSTEM")) {
				/* to do */
			}
			else {
				data = readFile(fname);
			}
			response.setContent(data);
			response.setCode(HttpConstants.HTTP_OK);
		} 
		catch (FileNotFoundException e) {
			response.setCode(HttpConstants.HTTP_NOT_FOUND);
			response.setComment("Not Found");
		} 
		catch (SecurityException e) {
			response.setCode(HttpConstants.HTTP_FORBIDDEN);
			response.setComment("Forbidden...not enough access rights");
		} 
		catch (IOException e) {
			response.setCode(HttpConstants.HTTP_SERVER_ERROR);
			response.setComment("I/O errors");
		}
		return response;
	}

	protected Response handleUnknownMethod(String method) {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("HTTP");
		response.setVersion("1.0");
		response.setCode(HttpConstants.HTTP_BAD_REQUEST);
		response.setComment("Unknown method: " + method);
		return response;
	}

	protected byte[] readFile(String name) throws FileNotFoundException,
			SecurityException, IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);

		if (name.startsWith("/")) {
			name = name.substring(1, name.length());
		}
		File f = new File(name);
		DataInputStream in = new DataInputStream(new FileInputStream(f));
		int len = (int) f.length();
		byte[] buf = new byte[len];
		in.readFully(buf);
		dos.write(buf, 0, len);
		in.close();
		dos.close();
		return baos.toByteArray();
	}

	public void setAgentRuntimeManager(AgentRuntimeManager manager) {
		this.manager = manager;
	}
}