/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol.matp;

import agentspace.network.protocol.Request;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseFactory;
import agentspace.runtime.AgentRuntimeManager;

public class SessionHandler implements agentspace.network.AgentProtocolHandler {
	Request request;
	AgentRuntimeManager manager;
	
	public SessionHandler() {
		super();
	}

	public void setRequest(Request request) {
		this.request = request;
	}

	public Response handle() {
		ResponseFactory response = new ResponseFactory();
		response.setProtocol("MATP");
		response.setVersion("1.0");
		String method = request.getMethod();
		try {
			if (method.equals("MOVE")) {
				byte[] content = request.getContent();
				manager.load(content);
				response.setCode(MatpConstants.MATP_OK);
				response.setComment("OK");
			}
			else if (method.equals("HELLO")) {
				response.setCode(MatpConstants.MATP_OK);
				response.setComment("OK");
			}
			else if (method.equals("PUT")) {
				byte[] content = request.getContent();
				manager.load(content);
				response.setCode(MatpConstants.MATP_OK);
				response.setComment("OK");
			}			
			else if (method.equals("GET")) {
//				byte[] content = manager.store(request.getURI());
				response.setCode(MatpConstants.MATP_OK);
				response.setComment("OK");
			}
			else if (method.equals("FETCH")) {
//				byte[] content = manager.store(request.getURI());
				response.setCode(MatpConstants.MATP_OK);
				response.setComment("OK");
			}
			else {
				response.setCode(MatpConstants.MATP_BAD_REQUEST);
				response.setComment("Unknown method: " + method);
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
			response.setCode(MatpConstants.MATP_SERVER_ERROR);
			response.setComment("Server errors");
		} 
		return response;
	}

	public void setAgentRuntimeManager(AgentRuntimeManager manager) {
		this.manager = manager;
	}
}