/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Copyright Ichiro Satoh, 1998, 1999, 2000, 2002, 2004, 2005, 2007.
 */

package agentspace.runtime.loader;

import java.util.Hashtable;

public class HashtableClassLoader extends ClassLoader {
    private Hashtable<String, Class> cache;
    private Hashtable codes;
    /* 
     * set hashtable that maintains pairs of class names and class data 
     */
    public HashtableClassLoader(Hashtable codes) {
    	this.codes = codes;
    	cache = new Hashtable<String, Class>();
    }

    public Class findClass(String name) throws ClassNotFoundException {
        return loadClass(name);
    }

    public synchronized Class loadClass(String name) throws ClassNotFoundException {
        return loadClass(name, true);
    }
    
    public Hashtable getCodes() {
    	return codes;	
    }
    
    public boolean contains(String name) {
    	return codes.containsKey(name);
    }

    /* 
     * @see java.lang.ClassLoader#loadClass(java.lang.String, boolean)
     */
    public synchronized Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
    	if (name == null) {
    		throw new ClassNotFoundException("Null class cannot be loaded.");
    	}
//    	System.err.println("HashClassLoader: name = "+name);
        Class newClass = cache.get(name);
        if (newClass == null) {
        	if (codes != null) {
        		byte[] data = (byte[])codes.get(name);
        		if (data != null) { 
        			try {
        				newClass = defineClass(name, data, 0, data.length);
            			cache.put(name, newClass);
        			}
        			catch (ClassFormatError ex){
        				newClass = null;
        			}
//        			System.err.println("HashClassLoader: name = "+name+" class "+newClass);
        		}
        	}
        }
        /*  if the class is not stored in the hashtable, a class specified as name in
         *  the default JDK class tree is loaded. 
         */  
        if (newClass == null) {
            newClass = findSystemClass(name);
			cache.put(name, newClass);
        }
        if (resolve) {
            resolveClass(newClass);
        }
        return newClass;
    }
}